// Generated by https://github.com/foxglove/schemas

export const ArrowPrimitive = {
  "title": "foxglove.ArrowPrimitive",
  "description": "A primitive representing an arrow",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Position of the arrow's tail and orientation of the arrow. Identity orientation means the arrow points in the +x direction.",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "shaft_length": {
      "type": "number",
      "description": "Length of the arrow shaft"
    },
    "shaft_diameter": {
      "type": "number",
      "description": "Diameter of the arrow shaft"
    },
    "head_length": {
      "type": "number",
      "description": "Length of the arrow head"
    },
    "head_diameter": {
      "type": "number",
      "description": "Diameter of the arrow head"
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Color of the arrow",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    }
  }
};

export const CameraCalibration = {
  "title": "foxglove.CameraCalibration",
  "description": "Camera calibration parameters",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of calibration data"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference for the camera. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image."
    },
    "width": {
      "type": "integer",
      "minimum": 0,
      "description": "Image width"
    },
    "height": {
      "type": "integer",
      "minimum": 0,
      "description": "Image height"
    },
    "distortion_model": {
      "type": "string",
      "description": "Name of distortion model\n\nSupported values: `plumb_bob` and `rational_polynomial`"
    },
    "D": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "description": "Distortion parameters"
    },
    "K": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "minItems": 9,
      "maxItems": 9,
      "description": "Intrinsic camera matrix (3x3 row-major matrix)\n\nA 3x3 row-major matrix for the raw (distorted) image.\n\nProjects 3D points in the camera coordinate frame to 2D pixel coordinates using the focal lengths (fx, fy) and principal point (cx, cy).\n\n```\n    [fx  0 cx]\nK = [ 0 fy cy]\n    [ 0  0  1]\n```\n"
    },
    "R": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "minItems": 9,
      "maxItems": 9,
      "description": "Rectification matrix (stereo cameras only, 3x3 row-major matrix)\n\nA rotation matrix aligning the camera coordinate system to the ideal stereo image plane so that epipolar lines in both stereo images are parallel."
    },
    "P": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "minItems": 12,
      "maxItems": 12,
      "description": "Projection/camera matrix (3x4 row-major matrix)\n\n```\n    [fx'  0  cx' Tx]\nP = [ 0  fy' cy' Ty]\n    [ 0   0   1   0]\n```\n\nBy convention, this matrix specifies the intrinsic (camera) matrix of the processed (rectified) image. That is, the left 3x3 portion is the normal camera intrinsic matrix for the rectified image.\n\nIt projects 3D points in the camera coordinate frame to 2D pixel coordinates using the focal lengths (fx', fy') and principal point (cx', cy') - these may differ from the values in K.\n\nFor monocular cameras, Tx = Ty = 0. Normally, monocular cameras will also have R = the identity and P[1:3,1:3] = K.\n\nFor a stereo pair, the fourth column [Tx Ty 0]' is related to the position of the optical center of the second camera in the first camera's frame. We assume Tz = 0 so both cameras are in the same stereo image plane. The first camera always has Tx = Ty = 0. For the right (second) camera of a horizontal stereo pair, Ty = 0 and Tx = -fx' * B, where B is the baseline between the cameras.\n\nGiven a 3D point [X Y Z]', the projection (x, y) of the point onto the rectified image is given by:\n\n```\n[u v w]' = P * [X Y Z 1]'\n       x = u / w\n       y = v / w\n```\n\nThis holds for both images of a stereo pair.\n"
    }
  }
};

export const CircleAnnotation = {
  "title": "foxglove.CircleAnnotation",
  "description": "A circle annotation on a 2D image",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of circle"
    },
    "position": {
      "title": "foxglove.Point2",
      "description": "Center of the circle in 2D image coordinates (pixels)",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate position"
        },
        "y": {
          "type": "number",
          "description": "y coordinate position"
        }
      }
    },
    "diameter": {
      "type": "number",
      "description": "Circle diameter in pixels"
    },
    "thickness": {
      "type": "number",
      "description": "Line thickness in pixels"
    },
    "fill_color": {
      "title": "foxglove.Color",
      "description": "Fill color",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "outline_color": {
      "title": "foxglove.Color",
      "description": "Outline color",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    }
  }
};

export const Color = {
  "title": "foxglove.Color",
  "description": "A color in RGBA format",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "r": {
      "type": "number",
      "description": "Red value between 0 and 1"
    },
    "g": {
      "type": "number",
      "description": "Green value between 0 and 1"
    },
    "b": {
      "type": "number",
      "description": "Blue value between 0 and 1"
    },
    "a": {
      "type": "number",
      "description": "Alpha value between 0 and 1"
    }
  }
};

export const CompressedImage = {
  "title": "foxglove.CompressedImage",
  "description": "A compressed image",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of image"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference for the image. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image."
    },
    "data": {
      "type": "string",
      "contentEncoding": "base64",
      "description": "Compressed image data"
    },
    "format": {
      "type": "string",
      "description": "Image format\n\nSupported values: `webp`, `jpeg`, `png`"
    }
  }
};

export const CylinderPrimitive = {
  "title": "foxglove.CylinderPrimitive",
  "description": "A primitive representing a cylinder, elliptic cylinder, or truncated cone",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Position of the center of the cylinder and orientation of the cylinder. The flat face(s) are perpendicular to the z-axis.",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "size": {
      "title": "foxglove.Vector3",
      "description": "Size of the cylinder's bounding box",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        },
        "z": {
          "type": "number",
          "description": "z coordinate length"
        }
      }
    },
    "bottom_scale": {
      "type": "number",
      "description": "0-1, ratio of the diameter of the cylinder's bottom face (min z) to the bottom of the bounding box"
    },
    "top_scale": {
      "type": "number",
      "description": "0-1, ratio of the diameter of the cylinder's top face (max z) to the top of the bounding box"
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Color of the cylinder",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    }
  }
};

export const CubePrimitive = {
  "title": "foxglove.CubePrimitive",
  "description": "A primitive representing a cube or rectangular prism",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Position of the center of the cube and orientation of the cube",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "size": {
      "title": "foxglove.Vector3",
      "description": "Size of the cube along each axis",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        },
        "z": {
          "type": "number",
          "description": "z coordinate length"
        }
      }
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Color of the cube",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    }
  }
};

export const FrameTransform = {
  "title": "foxglove.FrameTransform",
  "description": "A transform between two reference frames in 3D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of transform"
    },
    "parent_frame_id": {
      "type": "string",
      "description": "Name of the parent frame"
    },
    "child_frame_id": {
      "type": "string",
      "description": "Name of the child frame"
    },
    "translation": {
      "title": "foxglove.Vector3",
      "description": "Translation component of the transform",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        },
        "z": {
          "type": "number",
          "description": "z coordinate length"
        }
      }
    },
    "rotation": {
      "title": "foxglove.Quaternion",
      "description": "Rotation component of the transform",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x value"
        },
        "y": {
          "type": "number",
          "description": "y value"
        },
        "z": {
          "type": "number",
          "description": "z value"
        },
        "w": {
          "type": "number",
          "description": "w value"
        }
      }
    }
  }
};

export const FrameTransforms = {
  "title": "foxglove.FrameTransforms",
  "description": "An array of FrameTransform messages",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "transforms": {
      "type": "array",
      "items": {
        "title": "foxglove.FrameTransform",
        "description": "A transform between two reference frames in 3D space",
        "type": "object",
        "properties": {
          "timestamp": {
            "type": "object",
            "title": "time",
            "properties": {
              "sec": {
                "type": "integer",
                "minimum": 0
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Timestamp of transform"
          },
          "parent_frame_id": {
            "type": "string",
            "description": "Name of the parent frame"
          },
          "child_frame_id": {
            "type": "string",
            "description": "Name of the child frame"
          },
          "translation": {
            "title": "foxglove.Vector3",
            "description": "Translation component of the transform",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate length"
              },
              "y": {
                "type": "number",
                "description": "y coordinate length"
              },
              "z": {
                "type": "number",
                "description": "z coordinate length"
              }
            }
          },
          "rotation": {
            "title": "foxglove.Quaternion",
            "description": "Rotation component of the transform",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x value"
              },
              "y": {
                "type": "number",
                "description": "y value"
              },
              "z": {
                "type": "number",
                "description": "z value"
              },
              "w": {
                "type": "number",
                "description": "w value"
              }
            }
          }
        }
      },
      "description": "Array of transforms"
    }
  }
};

export const GeoJSON = {
  "title": "foxglove.GeoJSON",
  "description": "GeoJSON data for annotating maps",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "geojson": {
      "type": "string",
      "description": "GeoJSON data encoded as a UTF-8 string"
    }
  }
};

export const Grid = {
  "title": "foxglove.Grid",
  "description": "A 2D grid of data",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of grid"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference"
    },
    "pose": {
      "title": "foxglove.Pose",
      "description": "Origin of grid's corner relative to frame of reference; grid is positioned in the x-y plane relative to this origin",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "column_count": {
      "type": "integer",
      "minimum": 0,
      "description": "Number of grid columns"
    },
    "cell_size": {
      "title": "foxglove.Vector2",
      "description": "Size of single grid cell along x and y axes, relative to `pose`",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        }
      }
    },
    "row_stride": {
      "type": "integer",
      "minimum": 0,
      "description": "Number of bytes between rows in `data`"
    },
    "cell_stride": {
      "type": "integer",
      "minimum": 0,
      "description": "Number of bytes between cells within a row in `data`"
    },
    "fields": {
      "type": "array",
      "items": {
        "title": "foxglove.PackedElementField",
        "description": "A field present within each element in a byte array of packed elements.",
        "type": "object",
        "properties": {
          "name": {
            "type": "string",
            "description": "Name of the field"
          },
          "offset": {
            "type": "integer",
            "minimum": 0,
            "description": "Byte offset from start of data buffer"
          },
          "type": {
            "title": "foxglove.NumericType",
            "description": "Type of data in the field. Integers are stored using little-endian byte order.",
            "oneOf": [
              {
                "title": "UNKNOWN",
                "const": 0
              },
              {
                "title": "UINT8",
                "const": 1
              },
              {
                "title": "INT8",
                "const": 2
              },
              {
                "title": "UINT16",
                "const": 3
              },
              {
                "title": "INT16",
                "const": 4
              },
              {
                "title": "UINT32",
                "const": 5
              },
              {
                "title": "INT32",
                "const": 6
              },
              {
                "title": "FLOAT32",
                "const": 7
              },
              {
                "title": "FLOAT64",
                "const": 8
              }
            ]
          }
        }
      },
      "description": "Fields in `data`. `red`, `green`, `blue`, and `alpha` are optional for customizing the grid's color."
    },
    "data": {
      "type": "string",
      "contentEncoding": "base64",
      "description": "Grid cell data, interpreted using `fields`, in row-major (y-major) order"
    }
  }
};

export const ImageAnnotations = {
  "title": "foxglove.ImageAnnotations",
  "description": "Array of annotations for a 2D image",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "circles": {
      "type": "array",
      "items": {
        "title": "foxglove.CircleAnnotation",
        "description": "A circle annotation on a 2D image",
        "type": "object",
        "properties": {
          "timestamp": {
            "type": "object",
            "title": "time",
            "properties": {
              "sec": {
                "type": "integer",
                "minimum": 0
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Timestamp of circle"
          },
          "position": {
            "title": "foxglove.Point2",
            "description": "Center of the circle in 2D image coordinates (pixels)",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate position"
              },
              "y": {
                "type": "number",
                "description": "y coordinate position"
              }
            }
          },
          "diameter": {
            "type": "number",
            "description": "Circle diameter in pixels"
          },
          "thickness": {
            "type": "number",
            "description": "Line thickness in pixels"
          },
          "fill_color": {
            "title": "foxglove.Color",
            "description": "Fill color",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "outline_color": {
            "title": "foxglove.Color",
            "description": "Outline color",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          }
        }
      },
      "description": "Circle annotations"
    },
    "points": {
      "type": "array",
      "items": {
        "title": "foxglove.PointsAnnotation",
        "description": "An array of points on a 2D image",
        "type": "object",
        "properties": {
          "timestamp": {
            "type": "object",
            "title": "time",
            "properties": {
              "sec": {
                "type": "integer",
                "minimum": 0
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Timestamp of annotation"
          },
          "type": {
            "title": "foxglove.PointsAnnotationType",
            "description": "Type of points annotation to draw",
            "oneOf": [
              {
                "title": "UNKNOWN",
                "const": 0
              },
              {
                "title": "POINTS",
                "const": 1,
                "description": "Individual points: 0, 1, 2, ..."
              },
              {
                "title": "LINE_LOOP",
                "const": 2,
                "description": "Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0"
              },
              {
                "title": "LINE_STRIP",
                "const": 3,
                "description": "Connected line segments: 0-1, 1-2, ..., (n-1)-n"
              },
              {
                "title": "LINE_LIST",
                "const": 4,
                "description": "Individual line segments: 0-1, 2-3, 4-5, ..."
              }
            ]
          },
          "points": {
            "type": "array",
            "items": {
              "title": "foxglove.Point2",
              "description": "A point representing a position in 2D space",
              "type": "object",
              "properties": {
                "x": {
                  "type": "number",
                  "description": "x coordinate position"
                },
                "y": {
                  "type": "number",
                  "description": "y coordinate position"
                }
              }
            },
            "description": "Points in 2D image coordinates (pixels)"
          },
          "outline_color": {
            "title": "foxglove.Color",
            "description": "Outline color",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "outline_colors": {
            "type": "array",
            "items": {
              "title": "foxglove.Color",
              "description": "A color in RGBA format",
              "type": "object",
              "properties": {
                "r": {
                  "type": "number",
                  "description": "Red value between 0 and 1"
                },
                "g": {
                  "type": "number",
                  "description": "Green value between 0 and 1"
                },
                "b": {
                  "type": "number",
                  "description": "Blue value between 0 and 1"
                },
                "a": {
                  "type": "number",
                  "description": "Alpha value between 0 and 1"
                }
              }
            },
            "description": "Per-point colors, if `type` is `POINTS`, or per-segment stroke colors, if `type` is `LINE_LIST`."
          },
          "fill_color": {
            "title": "foxglove.Color",
            "description": "Fill color",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "thickness": {
            "type": "number",
            "description": "Stroke thickness in pixels"
          }
        }
      },
      "description": "Points annotations"
    },
    "texts": {
      "type": "array",
      "items": {
        "title": "foxglove.TextAnnotation",
        "description": "A text label on a 2D image",
        "type": "object",
        "properties": {
          "timestamp": {
            "type": "object",
            "title": "time",
            "properties": {
              "sec": {
                "type": "integer",
                "minimum": 0
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Timestamp of annotation"
          },
          "position": {
            "title": "foxglove.Point2",
            "description": "Bottom-left origin of the text label in 2D image coordinates (pixels)",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate position"
              },
              "y": {
                "type": "number",
                "description": "y coordinate position"
              }
            }
          },
          "text": {
            "type": "string",
            "description": "Text to display"
          },
          "font_size": {
            "type": "number",
            "description": "Font size in pixels"
          },
          "text_color": {
            "title": "foxglove.Color",
            "description": "Text color",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "background_color": {
            "title": "foxglove.Color",
            "description": "Background fill color",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          }
        }
      },
      "description": "Text annotations"
    }
  }
};

export const KeyValuePair = {
  "title": "foxglove.KeyValuePair",
  "description": "A key with its associated value",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "key": {
      "type": "string",
      "description": "Key"
    },
    "value": {
      "type": "string",
      "description": "Value"
    }
  }
};

export const LaserScan = {
  "title": "foxglove.LaserScan",
  "description": "A single scan from a planar laser range-finder",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of scan"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference"
    },
    "pose": {
      "title": "foxglove.Pose",
      "description": "Origin of scan relative to frame of reference; points are positioned in the x-y plane relative to this origin; angles are interpreted as counterclockwise rotations around the z axis with 0 rad being in the +x direction",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "start_angle": {
      "type": "number",
      "description": "Bearing of first point, in radians"
    },
    "end_angle": {
      "type": "number",
      "description": "Bearing of last point, in radians"
    },
    "ranges": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "description": "Distance of detections from origin; assumed to be at equally-spaced angles between `start_angle` and `end_angle`"
    },
    "intensities": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "description": "Intensity of detections"
    }
  }
};

export const LinePrimitive = {
  "title": "foxglove.LinePrimitive",
  "description": "A primitive representing a series of points connected by lines",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "type": {
      "title": "foxglove.LineType",
      "description": "Drawing primitive to use for lines",
      "oneOf": [
        {
          "title": "LINE_STRIP",
          "const": 0,
          "description": "Connected line segments: 0-1, 1-2, ..., (n-1)-n"
        },
        {
          "title": "LINE_LOOP",
          "const": 1,
          "description": "Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0"
        },
        {
          "title": "LINE_LIST",
          "const": 2,
          "description": "Individual line segments: 0-1, 2-3, 4-5, ..."
        }
      ]
    },
    "pose": {
      "title": "foxglove.Pose",
      "description": "Origin of lines relative to reference frame",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "thickness": {
      "type": "number",
      "description": "Line thickness"
    },
    "scale_invariant": {
      "type": "boolean",
      "description": "Indicates whether `thickness` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)"
    },
    "points": {
      "type": "array",
      "items": {
        "title": "foxglove.Point3",
        "description": "A point representing a position in 3D space",
        "type": "object",
        "properties": {
          "x": {
            "type": "number",
            "description": "x coordinate position"
          },
          "y": {
            "type": "number",
            "description": "y coordinate position"
          },
          "z": {
            "type": "number",
            "description": "z coordinate position"
          }
        }
      },
      "description": "Points along the line"
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Solid color to use for the whole line. One of `color` or `colors` must be provided.",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "colors": {
      "type": "array",
      "items": {
        "title": "foxglove.Color",
        "description": "A color in RGBA format",
        "type": "object",
        "properties": {
          "r": {
            "type": "number",
            "description": "Red value between 0 and 1"
          },
          "g": {
            "type": "number",
            "description": "Green value between 0 and 1"
          },
          "b": {
            "type": "number",
            "description": "Blue value between 0 and 1"
          },
          "a": {
            "type": "number",
            "description": "Alpha value between 0 and 1"
          }
        }
      },
      "description": "Per-point colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided."
    },
    "indices": {
      "type": "array",
      "items": {
        "type": "integer",
        "minimum": 0
      },
      "description": "Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.\n\nIf omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided)."
    }
  }
};

export const LocationFix = {
  "title": "foxglove.LocationFix",
  "description": "A navigation satellite fix for any Global Navigation Satellite System",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "latitude": {
      "type": "number",
      "description": "Latitude in degrees"
    },
    "longitude": {
      "type": "number",
      "description": "Longitude in degrees"
    },
    "altitude": {
      "type": "number",
      "description": "Altitude in meters"
    },
    "position_covariance": {
      "type": "array",
      "items": {
        "type": "number"
      },
      "minItems": 9,
      "maxItems": 9,
      "description": "Position covariance (m^2) defined relative to a tangential plane through the reported position. The components are East, North, and Up (ENU), in row-major order."
    },
    "position_covariance_type": {
      "title": "foxglove.PositionCovarianceType",
      "description": "If `position_covariance` is available, `position_covariance_type` must be set to indicate the type of covariance.",
      "oneOf": [
        {
          "title": "UNKNOWN",
          "const": 0
        },
        {
          "title": "APPROXIMATED",
          "const": 1
        },
        {
          "title": "DIAGONAL_KNOWN",
          "const": 2
        },
        {
          "title": "KNOWN",
          "const": 3
        }
      ]
    }
  }
};

export const Log = {
  "title": "foxglove.Log",
  "description": "A log message",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of log message"
    },
    "level": {
      "title": "foxglove.LogLevel",
      "description": "Log level",
      "oneOf": [
        {
          "title": "UNKNOWN",
          "const": 0
        },
        {
          "title": "DEBUG",
          "const": 1
        },
        {
          "title": "INFO",
          "const": 2
        },
        {
          "title": "WARNING",
          "const": 3
        },
        {
          "title": "ERROR",
          "const": 4
        },
        {
          "title": "FATAL",
          "const": 5
        }
      ]
    },
    "message": {
      "type": "string",
      "description": "Log message"
    },
    "name": {
      "type": "string",
      "description": "Process or node name"
    },
    "file": {
      "type": "string",
      "description": "Filename"
    },
    "line": {
      "type": "integer",
      "minimum": 0,
      "description": "Line number in the file"
    }
  }
};

export const SceneEntityDeletion = {
  "title": "foxglove.SceneEntityDeletion",
  "description": "Command to remove previously published entities",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of the deletion. Only matching entities earlier than this timestamp will be deleted."
    },
    "type": {
      "title": "foxglove.SceneEntityDeletionType",
      "description": "Type of deletion action to perform",
      "oneOf": [
        {
          "title": "MATCHING_ID",
          "const": 0,
          "description": "Delete the existing entity on the same topic that has the provided `id`"
        },
        {
          "title": "ALL",
          "const": 1,
          "description": "Delete all existing entities on the same topic"
        }
      ]
    },
    "id": {
      "type": "string",
      "description": "Identifier which must match if `type` is `MATCHING_ID`."
    }
  }
};

export const SceneEntity = {
  "title": "foxglove.SceneEntity",
  "description": "A visual element in a 3D scene. An entity may be composed of multiple primitives which all share the same frame of reference.",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of the entity"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference"
    },
    "id": {
      "type": "string",
      "description": "Identifier for the entity. A entity will replace any prior entity on the same topic with the same `id`."
    },
    "lifetime": {
      "type": "object",
      "title": "duration",
      "properties": {
        "sec": {
          "type": "integer"
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Length of time (relative to `timestamp`) after which the entity should be automatically removed. Zero value indicates the entity should remain visible until it is replaced or deleted."
    },
    "frame_locked": {
      "type": "boolean",
      "description": "Whether the entity should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)"
    },
    "metadata": {
      "type": "array",
      "items": {
        "title": "foxglove.KeyValuePair",
        "description": "A key with its associated value",
        "type": "object",
        "properties": {
          "key": {
            "type": "string",
            "description": "Key"
          },
          "value": {
            "type": "string",
            "description": "Value"
          }
        }
      },
      "description": "Additional user-provided metadata associated with the entity. Keys must be unique."
    },
    "arrows": {
      "type": "array",
      "items": {
        "title": "foxglove.ArrowPrimitive",
        "description": "A primitive representing an arrow",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Position of the arrow's tail and orientation of the arrow. Identity orientation means the arrow points in the +x direction.",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "shaft_length": {
            "type": "number",
            "description": "Length of the arrow shaft"
          },
          "shaft_diameter": {
            "type": "number",
            "description": "Diameter of the arrow shaft"
          },
          "head_length": {
            "type": "number",
            "description": "Length of the arrow head"
          },
          "head_diameter": {
            "type": "number",
            "description": "Diameter of the arrow head"
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Color of the arrow",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          }
        }
      },
      "description": "Arrow primitives"
    },
    "cubes": {
      "type": "array",
      "items": {
        "title": "foxglove.CubePrimitive",
        "description": "A primitive representing a cube or rectangular prism",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Position of the center of the cube and orientation of the cube",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "size": {
            "title": "foxglove.Vector3",
            "description": "Size of the cube along each axis",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate length"
              },
              "y": {
                "type": "number",
                "description": "y coordinate length"
              },
              "z": {
                "type": "number",
                "description": "z coordinate length"
              }
            }
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Color of the cube",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          }
        }
      },
      "description": "Cube primitives"
    },
    "spheres": {
      "type": "array",
      "items": {
        "title": "foxglove.SpherePrimitive",
        "description": "A primitive representing a sphere or ellipsoid",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Position of the center of the sphere and orientation of the sphere",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "size": {
            "title": "foxglove.Vector3",
            "description": "Size (diameter) of the sphere along each axis",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate length"
              },
              "y": {
                "type": "number",
                "description": "y coordinate length"
              },
              "z": {
                "type": "number",
                "description": "z coordinate length"
              }
            }
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Color of the sphere",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          }
        }
      },
      "description": "Sphere primitives"
    },
    "cylinders": {
      "type": "array",
      "items": {
        "title": "foxglove.CylinderPrimitive",
        "description": "A primitive representing a cylinder, elliptic cylinder, or truncated cone",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Position of the center of the cylinder and orientation of the cylinder. The flat face(s) are perpendicular to the z-axis.",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "size": {
            "title": "foxglove.Vector3",
            "description": "Size of the cylinder's bounding box",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate length"
              },
              "y": {
                "type": "number",
                "description": "y coordinate length"
              },
              "z": {
                "type": "number",
                "description": "z coordinate length"
              }
            }
          },
          "bottom_scale": {
            "type": "number",
            "description": "0-1, ratio of the diameter of the cylinder's bottom face (min z) to the bottom of the bounding box"
          },
          "top_scale": {
            "type": "number",
            "description": "0-1, ratio of the diameter of the cylinder's top face (max z) to the top of the bounding box"
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Color of the cylinder",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          }
        }
      },
      "description": "Cylinder primitives"
    },
    "lines": {
      "type": "array",
      "items": {
        "title": "foxglove.LinePrimitive",
        "description": "A primitive representing a series of points connected by lines",
        "type": "object",
        "properties": {
          "type": {
            "title": "foxglove.LineType",
            "description": "Drawing primitive to use for lines",
            "oneOf": [
              {
                "title": "LINE_STRIP",
                "const": 0,
                "description": "Connected line segments: 0-1, 1-2, ..., (n-1)-n"
              },
              {
                "title": "LINE_LOOP",
                "const": 1,
                "description": "Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0"
              },
              {
                "title": "LINE_LIST",
                "const": 2,
                "description": "Individual line segments: 0-1, 2-3, 4-5, ..."
              }
            ]
          },
          "pose": {
            "title": "foxglove.Pose",
            "description": "Origin of lines relative to reference frame",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "thickness": {
            "type": "number",
            "description": "Line thickness"
          },
          "scale_invariant": {
            "type": "boolean",
            "description": "Indicates whether `thickness` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)"
          },
          "points": {
            "type": "array",
            "items": {
              "title": "foxglove.Point3",
              "description": "A point representing a position in 3D space",
              "type": "object",
              "properties": {
                "x": {
                  "type": "number",
                  "description": "x coordinate position"
                },
                "y": {
                  "type": "number",
                  "description": "y coordinate position"
                },
                "z": {
                  "type": "number",
                  "description": "z coordinate position"
                }
              }
            },
            "description": "Points along the line"
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Solid color to use for the whole line. One of `color` or `colors` must be provided.",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "colors": {
            "type": "array",
            "items": {
              "title": "foxglove.Color",
              "description": "A color in RGBA format",
              "type": "object",
              "properties": {
                "r": {
                  "type": "number",
                  "description": "Red value between 0 and 1"
                },
                "g": {
                  "type": "number",
                  "description": "Green value between 0 and 1"
                },
                "b": {
                  "type": "number",
                  "description": "Blue value between 0 and 1"
                },
                "a": {
                  "type": "number",
                  "description": "Alpha value between 0 and 1"
                }
              }
            },
            "description": "Per-point colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided."
          },
          "indices": {
            "type": "array",
            "items": {
              "type": "integer",
              "minimum": 0
            },
            "description": "Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.\n\nIf omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided)."
          }
        }
      },
      "description": "Line primitives"
    },
    "triangles": {
      "type": "array",
      "items": {
        "title": "foxglove.TriangleListPrimitive",
        "description": "A primitive representing a set of triangles or a surface tiled by triangles",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Origin of triangles relative to reference frame",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "points": {
            "type": "array",
            "items": {
              "title": "foxglove.Point3",
              "description": "A point representing a position in 3D space",
              "type": "object",
              "properties": {
                "x": {
                  "type": "number",
                  "description": "x coordinate position"
                },
                "y": {
                  "type": "number",
                  "description": "y coordinate position"
                },
                "z": {
                  "type": "number",
                  "description": "z coordinate position"
                }
              }
            },
            "description": "Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)"
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Solid color to use for the whole shape. One of `color` or `colors` must be provided.",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "colors": {
            "type": "array",
            "items": {
              "title": "foxglove.Color",
              "description": "A color in RGBA format",
              "type": "object",
              "properties": {
                "r": {
                  "type": "number",
                  "description": "Red value between 0 and 1"
                },
                "g": {
                  "type": "number",
                  "description": "Green value between 0 and 1"
                },
                "b": {
                  "type": "number",
                  "description": "Blue value between 0 and 1"
                },
                "a": {
                  "type": "number",
                  "description": "Alpha value between 0 and 1"
                }
              }
            },
            "description": "Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided."
          },
          "indices": {
            "type": "array",
            "items": {
              "type": "integer",
              "minimum": 0
            },
            "description": "Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.\n\nIf omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided)."
          }
        }
      },
      "description": "Triangle list primitives"
    },
    "texts": {
      "type": "array",
      "items": {
        "title": "foxglove.TextPrimitive",
        "description": "A primitive representing a text label",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Position of the center of the text box and orientation of the text. Identity orientation means the text is oriented in the xy-plane and flows from -x to +x.",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "billboard": {
            "type": "boolean",
            "description": "Whether the text should respect `pose.orientation` (false) or always face the camera (true)"
          },
          "font_size": {
            "type": "number",
            "description": "Font size (height of one line of text)"
          },
          "scale_invariant": {
            "type": "boolean",
            "description": "Indicates whether `font_size` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)"
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Color of the text",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "text": {
            "type": "string",
            "description": "Text"
          }
        }
      },
      "description": "Text primitives"
    },
    "models": {
      "type": "array",
      "items": {
        "title": "foxglove.ModelPrimitive",
        "description": "A primitive representing a 3D model file loaded from an external URL or embedded data",
        "type": "object",
        "properties": {
          "pose": {
            "title": "foxglove.Pose",
            "description": "Origin of model relative to reference frame",
            "type": "object",
            "properties": {
              "position": {
                "title": "foxglove.Vector3",
                "description": "Point denoting position in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x coordinate length"
                  },
                  "y": {
                    "type": "number",
                    "description": "y coordinate length"
                  },
                  "z": {
                    "type": "number",
                    "description": "z coordinate length"
                  }
                }
              },
              "orientation": {
                "title": "foxglove.Quaternion",
                "description": "Quaternion denoting orientation in 3D space",
                "type": "object",
                "properties": {
                  "x": {
                    "type": "number",
                    "description": "x value"
                  },
                  "y": {
                    "type": "number",
                    "description": "y value"
                  },
                  "z": {
                    "type": "number",
                    "description": "z value"
                  },
                  "w": {
                    "type": "number",
                    "description": "w value"
                  }
                }
              }
            }
          },
          "scale": {
            "title": "foxglove.Vector3",
            "description": "Scale factor to apply to the model along each axis",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate length"
              },
              "y": {
                "type": "number",
                "description": "y coordinate length"
              },
              "z": {
                "type": "number",
                "description": "z coordinate length"
              }
            }
          },
          "color": {
            "title": "foxglove.Color",
            "description": "Solid color to use for the whole model if `override_color` is true.",
            "type": "object",
            "properties": {
              "r": {
                "type": "number",
                "description": "Red value between 0 and 1"
              },
              "g": {
                "type": "number",
                "description": "Green value between 0 and 1"
              },
              "b": {
                "type": "number",
                "description": "Blue value between 0 and 1"
              },
              "a": {
                "type": "number",
                "description": "Alpha value between 0 and 1"
              }
            }
          },
          "override_color": {
            "type": "boolean",
            "description": "Whether to use the color specified in `color` instead of any materials embedded in the original model."
          },
          "url": {
            "type": "string",
            "description": "URL pointing to model file. One of `url` or `data` should be provided."
          },
          "media_type": {
            "type": "string",
            "description": "[Media type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types) of embedded model (e.g. `model/gltf-binary`). Required if `data` is provided instead of `url`. Overrides the inferred media type if `url` is provided."
          },
          "data": {
            "type": "string",
            "contentEncoding": "base64",
            "description": "Embedded model. One of `url` or `data` should be provided. If `data` is provided, `media_type` must be set to indicate the type of the data."
          }
        }
      },
      "description": "Model primitives"
    }
  }
};

export const SceneUpdate = {
  "title": "foxglove.SceneUpdate",
  "description": "An update to the entities displayed in a 3D scene",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "deletions": {
      "type": "array",
      "items": {
        "title": "foxglove.SceneEntityDeletion",
        "description": "Command to remove previously published entities",
        "type": "object",
        "properties": {
          "timestamp": {
            "type": "object",
            "title": "time",
            "properties": {
              "sec": {
                "type": "integer",
                "minimum": 0
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Timestamp of the deletion. Only matching entities earlier than this timestamp will be deleted."
          },
          "type": {
            "title": "foxglove.SceneEntityDeletionType",
            "description": "Type of deletion action to perform",
            "oneOf": [
              {
                "title": "MATCHING_ID",
                "const": 0,
                "description": "Delete the existing entity on the same topic that has the provided `id`"
              },
              {
                "title": "ALL",
                "const": 1,
                "description": "Delete all existing entities on the same topic"
              }
            ]
          },
          "id": {
            "type": "string",
            "description": "Identifier which must match if `type` is `MATCHING_ID`."
          }
        }
      },
      "description": "Scene entities to delete"
    },
    "entities": {
      "type": "array",
      "items": {
        "title": "foxglove.SceneEntity",
        "description": "A visual element in a 3D scene. An entity may be composed of multiple primitives which all share the same frame of reference.",
        "type": "object",
        "properties": {
          "timestamp": {
            "type": "object",
            "title": "time",
            "properties": {
              "sec": {
                "type": "integer",
                "minimum": 0
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Timestamp of the entity"
          },
          "frame_id": {
            "type": "string",
            "description": "Frame of reference"
          },
          "id": {
            "type": "string",
            "description": "Identifier for the entity. A entity will replace any prior entity on the same topic with the same `id`."
          },
          "lifetime": {
            "type": "object",
            "title": "duration",
            "properties": {
              "sec": {
                "type": "integer"
              },
              "nsec": {
                "type": "integer",
                "minimum": 0,
                "maximum": 999999999
              }
            },
            "description": "Length of time (relative to `timestamp`) after which the entity should be automatically removed. Zero value indicates the entity should remain visible until it is replaced or deleted."
          },
          "frame_locked": {
            "type": "boolean",
            "description": "Whether the entity should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)"
          },
          "metadata": {
            "type": "array",
            "items": {
              "title": "foxglove.KeyValuePair",
              "description": "A key with its associated value",
              "type": "object",
              "properties": {
                "key": {
                  "type": "string",
                  "description": "Key"
                },
                "value": {
                  "type": "string",
                  "description": "Value"
                }
              }
            },
            "description": "Additional user-provided metadata associated with the entity. Keys must be unique."
          },
          "arrows": {
            "type": "array",
            "items": {
              "title": "foxglove.ArrowPrimitive",
              "description": "A primitive representing an arrow",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Position of the arrow's tail and orientation of the arrow. Identity orientation means the arrow points in the +x direction.",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "shaft_length": {
                  "type": "number",
                  "description": "Length of the arrow shaft"
                },
                "shaft_diameter": {
                  "type": "number",
                  "description": "Diameter of the arrow shaft"
                },
                "head_length": {
                  "type": "number",
                  "description": "Length of the arrow head"
                },
                "head_diameter": {
                  "type": "number",
                  "description": "Diameter of the arrow head"
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Color of the arrow",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                }
              }
            },
            "description": "Arrow primitives"
          },
          "cubes": {
            "type": "array",
            "items": {
              "title": "foxglove.CubePrimitive",
              "description": "A primitive representing a cube or rectangular prism",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Position of the center of the cube and orientation of the cube",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "size": {
                  "title": "foxglove.Vector3",
                  "description": "Size of the cube along each axis",
                  "type": "object",
                  "properties": {
                    "x": {
                      "type": "number",
                      "description": "x coordinate length"
                    },
                    "y": {
                      "type": "number",
                      "description": "y coordinate length"
                    },
                    "z": {
                      "type": "number",
                      "description": "z coordinate length"
                    }
                  }
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Color of the cube",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                }
              }
            },
            "description": "Cube primitives"
          },
          "spheres": {
            "type": "array",
            "items": {
              "title": "foxglove.SpherePrimitive",
              "description": "A primitive representing a sphere or ellipsoid",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Position of the center of the sphere and orientation of the sphere",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "size": {
                  "title": "foxglove.Vector3",
                  "description": "Size (diameter) of the sphere along each axis",
                  "type": "object",
                  "properties": {
                    "x": {
                      "type": "number",
                      "description": "x coordinate length"
                    },
                    "y": {
                      "type": "number",
                      "description": "y coordinate length"
                    },
                    "z": {
                      "type": "number",
                      "description": "z coordinate length"
                    }
                  }
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Color of the sphere",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                }
              }
            },
            "description": "Sphere primitives"
          },
          "cylinders": {
            "type": "array",
            "items": {
              "title": "foxglove.CylinderPrimitive",
              "description": "A primitive representing a cylinder, elliptic cylinder, or truncated cone",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Position of the center of the cylinder and orientation of the cylinder. The flat face(s) are perpendicular to the z-axis.",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "size": {
                  "title": "foxglove.Vector3",
                  "description": "Size of the cylinder's bounding box",
                  "type": "object",
                  "properties": {
                    "x": {
                      "type": "number",
                      "description": "x coordinate length"
                    },
                    "y": {
                      "type": "number",
                      "description": "y coordinate length"
                    },
                    "z": {
                      "type": "number",
                      "description": "z coordinate length"
                    }
                  }
                },
                "bottom_scale": {
                  "type": "number",
                  "description": "0-1, ratio of the diameter of the cylinder's bottom face (min z) to the bottom of the bounding box"
                },
                "top_scale": {
                  "type": "number",
                  "description": "0-1, ratio of the diameter of the cylinder's top face (max z) to the top of the bounding box"
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Color of the cylinder",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                }
              }
            },
            "description": "Cylinder primitives"
          },
          "lines": {
            "type": "array",
            "items": {
              "title": "foxglove.LinePrimitive",
              "description": "A primitive representing a series of points connected by lines",
              "type": "object",
              "properties": {
                "type": {
                  "title": "foxglove.LineType",
                  "description": "Drawing primitive to use for lines",
                  "oneOf": [
                    {
                      "title": "LINE_STRIP",
                      "const": 0,
                      "description": "Connected line segments: 0-1, 1-2, ..., (n-1)-n"
                    },
                    {
                      "title": "LINE_LOOP",
                      "const": 1,
                      "description": "Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0"
                    },
                    {
                      "title": "LINE_LIST",
                      "const": 2,
                      "description": "Individual line segments: 0-1, 2-3, 4-5, ..."
                    }
                  ]
                },
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Origin of lines relative to reference frame",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "thickness": {
                  "type": "number",
                  "description": "Line thickness"
                },
                "scale_invariant": {
                  "type": "boolean",
                  "description": "Indicates whether `thickness` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)"
                },
                "points": {
                  "type": "array",
                  "items": {
                    "title": "foxglove.Point3",
                    "description": "A point representing a position in 3D space",
                    "type": "object",
                    "properties": {
                      "x": {
                        "type": "number",
                        "description": "x coordinate position"
                      },
                      "y": {
                        "type": "number",
                        "description": "y coordinate position"
                      },
                      "z": {
                        "type": "number",
                        "description": "z coordinate position"
                      }
                    }
                  },
                  "description": "Points along the line"
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Solid color to use for the whole line. One of `color` or `colors` must be provided.",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                },
                "colors": {
                  "type": "array",
                  "items": {
                    "title": "foxglove.Color",
                    "description": "A color in RGBA format",
                    "type": "object",
                    "properties": {
                      "r": {
                        "type": "number",
                        "description": "Red value between 0 and 1"
                      },
                      "g": {
                        "type": "number",
                        "description": "Green value between 0 and 1"
                      },
                      "b": {
                        "type": "number",
                        "description": "Blue value between 0 and 1"
                      },
                      "a": {
                        "type": "number",
                        "description": "Alpha value between 0 and 1"
                      }
                    }
                  },
                  "description": "Per-point colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided."
                },
                "indices": {
                  "type": "array",
                  "items": {
                    "type": "integer",
                    "minimum": 0
                  },
                  "description": "Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.\n\nIf omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided)."
                }
              }
            },
            "description": "Line primitives"
          },
          "triangles": {
            "type": "array",
            "items": {
              "title": "foxglove.TriangleListPrimitive",
              "description": "A primitive representing a set of triangles or a surface tiled by triangles",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Origin of triangles relative to reference frame",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "points": {
                  "type": "array",
                  "items": {
                    "title": "foxglove.Point3",
                    "description": "A point representing a position in 3D space",
                    "type": "object",
                    "properties": {
                      "x": {
                        "type": "number",
                        "description": "x coordinate position"
                      },
                      "y": {
                        "type": "number",
                        "description": "y coordinate position"
                      },
                      "z": {
                        "type": "number",
                        "description": "z coordinate position"
                      }
                    }
                  },
                  "description": "Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)"
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Solid color to use for the whole shape. One of `color` or `colors` must be provided.",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                },
                "colors": {
                  "type": "array",
                  "items": {
                    "title": "foxglove.Color",
                    "description": "A color in RGBA format",
                    "type": "object",
                    "properties": {
                      "r": {
                        "type": "number",
                        "description": "Red value between 0 and 1"
                      },
                      "g": {
                        "type": "number",
                        "description": "Green value between 0 and 1"
                      },
                      "b": {
                        "type": "number",
                        "description": "Blue value between 0 and 1"
                      },
                      "a": {
                        "type": "number",
                        "description": "Alpha value between 0 and 1"
                      }
                    }
                  },
                  "description": "Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided."
                },
                "indices": {
                  "type": "array",
                  "items": {
                    "type": "integer",
                    "minimum": 0
                  },
                  "description": "Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.\n\nIf omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided)."
                }
              }
            },
            "description": "Triangle list primitives"
          },
          "texts": {
            "type": "array",
            "items": {
              "title": "foxglove.TextPrimitive",
              "description": "A primitive representing a text label",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Position of the center of the text box and orientation of the text. Identity orientation means the text is oriented in the xy-plane and flows from -x to +x.",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "billboard": {
                  "type": "boolean",
                  "description": "Whether the text should respect `pose.orientation` (false) or always face the camera (true)"
                },
                "font_size": {
                  "type": "number",
                  "description": "Font size (height of one line of text)"
                },
                "scale_invariant": {
                  "type": "boolean",
                  "description": "Indicates whether `font_size` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)"
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Color of the text",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                },
                "text": {
                  "type": "string",
                  "description": "Text"
                }
              }
            },
            "description": "Text primitives"
          },
          "models": {
            "type": "array",
            "items": {
              "title": "foxglove.ModelPrimitive",
              "description": "A primitive representing a 3D model file loaded from an external URL or embedded data",
              "type": "object",
              "properties": {
                "pose": {
                  "title": "foxglove.Pose",
                  "description": "Origin of model relative to reference frame",
                  "type": "object",
                  "properties": {
                    "position": {
                      "title": "foxglove.Vector3",
                      "description": "Point denoting position in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x coordinate length"
                        },
                        "y": {
                          "type": "number",
                          "description": "y coordinate length"
                        },
                        "z": {
                          "type": "number",
                          "description": "z coordinate length"
                        }
                      }
                    },
                    "orientation": {
                      "title": "foxglove.Quaternion",
                      "description": "Quaternion denoting orientation in 3D space",
                      "type": "object",
                      "properties": {
                        "x": {
                          "type": "number",
                          "description": "x value"
                        },
                        "y": {
                          "type": "number",
                          "description": "y value"
                        },
                        "z": {
                          "type": "number",
                          "description": "z value"
                        },
                        "w": {
                          "type": "number",
                          "description": "w value"
                        }
                      }
                    }
                  }
                },
                "scale": {
                  "title": "foxglove.Vector3",
                  "description": "Scale factor to apply to the model along each axis",
                  "type": "object",
                  "properties": {
                    "x": {
                      "type": "number",
                      "description": "x coordinate length"
                    },
                    "y": {
                      "type": "number",
                      "description": "y coordinate length"
                    },
                    "z": {
                      "type": "number",
                      "description": "z coordinate length"
                    }
                  }
                },
                "color": {
                  "title": "foxglove.Color",
                  "description": "Solid color to use for the whole model if `override_color` is true.",
                  "type": "object",
                  "properties": {
                    "r": {
                      "type": "number",
                      "description": "Red value between 0 and 1"
                    },
                    "g": {
                      "type": "number",
                      "description": "Green value between 0 and 1"
                    },
                    "b": {
                      "type": "number",
                      "description": "Blue value between 0 and 1"
                    },
                    "a": {
                      "type": "number",
                      "description": "Alpha value between 0 and 1"
                    }
                  }
                },
                "override_color": {
                  "type": "boolean",
                  "description": "Whether to use the color specified in `color` instead of any materials embedded in the original model."
                },
                "url": {
                  "type": "string",
                  "description": "URL pointing to model file. One of `url` or `data` should be provided."
                },
                "media_type": {
                  "type": "string",
                  "description": "[Media type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types) of embedded model (e.g. `model/gltf-binary`). Required if `data` is provided instead of `url`. Overrides the inferred media type if `url` is provided."
                },
                "data": {
                  "type": "string",
                  "contentEncoding": "base64",
                  "description": "Embedded model. One of `url` or `data` should be provided. If `data` is provided, `media_type` must be set to indicate the type of the data."
                }
              }
            },
            "description": "Model primitives"
          }
        }
      },
      "description": "Scene entities to add or replace"
    }
  }
};

export const ModelPrimitive = {
  "title": "foxglove.ModelPrimitive",
  "description": "A primitive representing a 3D model file loaded from an external URL or embedded data",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Origin of model relative to reference frame",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "scale": {
      "title": "foxglove.Vector3",
      "description": "Scale factor to apply to the model along each axis",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        },
        "z": {
          "type": "number",
          "description": "z coordinate length"
        }
      }
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Solid color to use for the whole model if `override_color` is true.",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "override_color": {
      "type": "boolean",
      "description": "Whether to use the color specified in `color` instead of any materials embedded in the original model."
    },
    "url": {
      "type": "string",
      "description": "URL pointing to model file. One of `url` or `data` should be provided."
    },
    "media_type": {
      "type": "string",
      "description": "[Media type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types) of embedded model (e.g. `model/gltf-binary`). Required if `data` is provided instead of `url`. Overrides the inferred media type if `url` is provided."
    },
    "data": {
      "type": "string",
      "contentEncoding": "base64",
      "description": "Embedded model. One of `url` or `data` should be provided. If `data` is provided, `media_type` must be set to indicate the type of the data."
    }
  }
};

export const PackedElementField = {
  "title": "foxglove.PackedElementField",
  "description": "A field present within each element in a byte array of packed elements.",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "name": {
      "type": "string",
      "description": "Name of the field"
    },
    "offset": {
      "type": "integer",
      "minimum": 0,
      "description": "Byte offset from start of data buffer"
    },
    "type": {
      "title": "foxglove.NumericType",
      "description": "Type of data in the field. Integers are stored using little-endian byte order.",
      "oneOf": [
        {
          "title": "UNKNOWN",
          "const": 0
        },
        {
          "title": "UINT8",
          "const": 1
        },
        {
          "title": "INT8",
          "const": 2
        },
        {
          "title": "UINT16",
          "const": 3
        },
        {
          "title": "INT16",
          "const": 4
        },
        {
          "title": "UINT32",
          "const": 5
        },
        {
          "title": "INT32",
          "const": 6
        },
        {
          "title": "FLOAT32",
          "const": 7
        },
        {
          "title": "FLOAT64",
          "const": 8
        }
      ]
    }
  }
};

export const Point2 = {
  "title": "foxglove.Point2",
  "description": "A point representing a position in 2D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "x": {
      "type": "number",
      "description": "x coordinate position"
    },
    "y": {
      "type": "number",
      "description": "y coordinate position"
    }
  }
};

export const Point3 = {
  "title": "foxglove.Point3",
  "description": "A point representing a position in 3D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "x": {
      "type": "number",
      "description": "x coordinate position"
    },
    "y": {
      "type": "number",
      "description": "y coordinate position"
    },
    "z": {
      "type": "number",
      "description": "z coordinate position"
    }
  }
};

export const PointCloud = {
  "title": "foxglove.PointCloud",
  "description": "A collection of N-dimensional points, which may contain additional fields with information like normals, intensity, etc.",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of point cloud"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference"
    },
    "pose": {
      "title": "foxglove.Pose",
      "description": "The origin of the point cloud relative to the frame of reference",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "point_stride": {
      "type": "integer",
      "minimum": 0,
      "description": "Number of bytes between points in the `data`"
    },
    "fields": {
      "type": "array",
      "items": {
        "title": "foxglove.PackedElementField",
        "description": "A field present within each element in a byte array of packed elements.",
        "type": "object",
        "properties": {
          "name": {
            "type": "string",
            "description": "Name of the field"
          },
          "offset": {
            "type": "integer",
            "minimum": 0,
            "description": "Byte offset from start of data buffer"
          },
          "type": {
            "title": "foxglove.NumericType",
            "description": "Type of data in the field. Integers are stored using little-endian byte order.",
            "oneOf": [
              {
                "title": "UNKNOWN",
                "const": 0
              },
              {
                "title": "UINT8",
                "const": 1
              },
              {
                "title": "INT8",
                "const": 2
              },
              {
                "title": "UINT16",
                "const": 3
              },
              {
                "title": "INT16",
                "const": 4
              },
              {
                "title": "UINT32",
                "const": 5
              },
              {
                "title": "INT32",
                "const": 6
              },
              {
                "title": "FLOAT32",
                "const": 7
              },
              {
                "title": "FLOAT64",
                "const": 8
              }
            ]
          }
        }
      },
      "description": "Fields in `data`. At least 2 coordinate fields from `x`, `y`, and `z` are required for each point's position; `red`, `green`, `blue`, and `alpha` are optional for customizing each point's color."
    },
    "data": {
      "type": "string",
      "contentEncoding": "base64",
      "description": "Point data, interpreted using `fields`"
    }
  }
};

export const PointsAnnotation = {
  "title": "foxglove.PointsAnnotation",
  "description": "An array of points on a 2D image",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of annotation"
    },
    "type": {
      "title": "foxglove.PointsAnnotationType",
      "description": "Type of points annotation to draw",
      "oneOf": [
        {
          "title": "UNKNOWN",
          "const": 0
        },
        {
          "title": "POINTS",
          "const": 1,
          "description": "Individual points: 0, 1, 2, ..."
        },
        {
          "title": "LINE_LOOP",
          "const": 2,
          "description": "Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0"
        },
        {
          "title": "LINE_STRIP",
          "const": 3,
          "description": "Connected line segments: 0-1, 1-2, ..., (n-1)-n"
        },
        {
          "title": "LINE_LIST",
          "const": 4,
          "description": "Individual line segments: 0-1, 2-3, 4-5, ..."
        }
      ]
    },
    "points": {
      "type": "array",
      "items": {
        "title": "foxglove.Point2",
        "description": "A point representing a position in 2D space",
        "type": "object",
        "properties": {
          "x": {
            "type": "number",
            "description": "x coordinate position"
          },
          "y": {
            "type": "number",
            "description": "y coordinate position"
          }
        }
      },
      "description": "Points in 2D image coordinates (pixels)"
    },
    "outline_color": {
      "title": "foxglove.Color",
      "description": "Outline color",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "outline_colors": {
      "type": "array",
      "items": {
        "title": "foxglove.Color",
        "description": "A color in RGBA format",
        "type": "object",
        "properties": {
          "r": {
            "type": "number",
            "description": "Red value between 0 and 1"
          },
          "g": {
            "type": "number",
            "description": "Green value between 0 and 1"
          },
          "b": {
            "type": "number",
            "description": "Blue value between 0 and 1"
          },
          "a": {
            "type": "number",
            "description": "Alpha value between 0 and 1"
          }
        }
      },
      "description": "Per-point colors, if `type` is `POINTS`, or per-segment stroke colors, if `type` is `LINE_LIST`."
    },
    "fill_color": {
      "title": "foxglove.Color",
      "description": "Fill color",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "thickness": {
      "type": "number",
      "description": "Stroke thickness in pixels"
    }
  }
};

export const Pose = {
  "title": "foxglove.Pose",
  "description": "A position and orientation for an object or reference frame in 3D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "position": {
      "title": "foxglove.Vector3",
      "description": "Point denoting position in 3D space",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        },
        "z": {
          "type": "number",
          "description": "z coordinate length"
        }
      }
    },
    "orientation": {
      "title": "foxglove.Quaternion",
      "description": "Quaternion denoting orientation in 3D space",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x value"
        },
        "y": {
          "type": "number",
          "description": "y value"
        },
        "z": {
          "type": "number",
          "description": "z value"
        },
        "w": {
          "type": "number",
          "description": "w value"
        }
      }
    }
  }
};

export const PoseInFrame = {
  "title": "foxglove.PoseInFrame",
  "description": "A timestamped pose for an object or reference frame in 3D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of pose"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference for pose position and orientation"
    },
    "pose": {
      "title": "foxglove.Pose",
      "description": "Pose in 3D space",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    }
  }
};

export const PosesInFrame = {
  "title": "foxglove.PosesInFrame",
  "description": "An array of timestamped poses for an object or reference frame in 3D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of pose"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference for pose position and orientation"
    },
    "poses": {
      "type": "array",
      "items": {
        "title": "foxglove.Pose",
        "description": "A position and orientation for an object or reference frame in 3D space",
        "type": "object",
        "properties": {
          "position": {
            "title": "foxglove.Vector3",
            "description": "Point denoting position in 3D space",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x coordinate length"
              },
              "y": {
                "type": "number",
                "description": "y coordinate length"
              },
              "z": {
                "type": "number",
                "description": "z coordinate length"
              }
            }
          },
          "orientation": {
            "title": "foxglove.Quaternion",
            "description": "Quaternion denoting orientation in 3D space",
            "type": "object",
            "properties": {
              "x": {
                "type": "number",
                "description": "x value"
              },
              "y": {
                "type": "number",
                "description": "y value"
              },
              "z": {
                "type": "number",
                "description": "z value"
              },
              "w": {
                "type": "number",
                "description": "w value"
              }
            }
          }
        }
      },
      "description": "Poses in 3D space"
    }
  }
};

export const Quaternion = {
  "title": "foxglove.Quaternion",
  "description": "A [quaternion](https://eater.net/quaternions) representing a rotation in 3D space",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "x": {
      "type": "number",
      "description": "x value"
    },
    "y": {
      "type": "number",
      "description": "y value"
    },
    "z": {
      "type": "number",
      "description": "z value"
    },
    "w": {
      "type": "number",
      "description": "w value"
    }
  }
};

export const RawImage = {
  "title": "foxglove.RawImage",
  "description": "A raw image",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of image"
    },
    "frame_id": {
      "type": "string",
      "description": "Frame of reference for the image. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image."
    },
    "width": {
      "type": "integer",
      "minimum": 0,
      "description": "Image width"
    },
    "height": {
      "type": "integer",
      "minimum": 0,
      "description": "Image height"
    },
    "encoding": {
      "type": "string",
      "description": "Encoding of the raw image data\n\nSupported values: `8UC1`, `8UC3`, `16UC1`, `32FC1`, `bayer_bggr8`, `bayer_gbrg8`, `bayer_grbg8`, `bayer_rggb8`, `bgr8`, `bgra8`, `mono8`, `mono16`, `rgb8`, `rgba8`, `yuv422`"
    },
    "step": {
      "type": "integer",
      "minimum": 0,
      "description": "Byte length of a single row"
    },
    "data": {
      "type": "string",
      "contentEncoding": "base64",
      "description": "Raw image data"
    }
  }
};

export const SpherePrimitive = {
  "title": "foxglove.SpherePrimitive",
  "description": "A primitive representing a sphere or ellipsoid",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Position of the center of the sphere and orientation of the sphere",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "size": {
      "title": "foxglove.Vector3",
      "description": "Size (diameter) of the sphere along each axis",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate length"
        },
        "y": {
          "type": "number",
          "description": "y coordinate length"
        },
        "z": {
          "type": "number",
          "description": "z coordinate length"
        }
      }
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Color of the sphere",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    }
  }
};

export const TextAnnotation = {
  "title": "foxglove.TextAnnotation",
  "description": "A text label on a 2D image",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "timestamp": {
      "type": "object",
      "title": "time",
      "properties": {
        "sec": {
          "type": "integer",
          "minimum": 0
        },
        "nsec": {
          "type": "integer",
          "minimum": 0,
          "maximum": 999999999
        }
      },
      "description": "Timestamp of annotation"
    },
    "position": {
      "title": "foxglove.Point2",
      "description": "Bottom-left origin of the text label in 2D image coordinates (pixels)",
      "type": "object",
      "properties": {
        "x": {
          "type": "number",
          "description": "x coordinate position"
        },
        "y": {
          "type": "number",
          "description": "y coordinate position"
        }
      }
    },
    "text": {
      "type": "string",
      "description": "Text to display"
    },
    "font_size": {
      "type": "number",
      "description": "Font size in pixels"
    },
    "text_color": {
      "title": "foxglove.Color",
      "description": "Text color",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "background_color": {
      "title": "foxglove.Color",
      "description": "Background fill color",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    }
  }
};

export const TextPrimitive = {
  "title": "foxglove.TextPrimitive",
  "description": "A primitive representing a text label",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Position of the center of the text box and orientation of the text. Identity orientation means the text is oriented in the xy-plane and flows from -x to +x.",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "billboard": {
      "type": "boolean",
      "description": "Whether the text should respect `pose.orientation` (false) or always face the camera (true)"
    },
    "font_size": {
      "type": "number",
      "description": "Font size (height of one line of text)"
    },
    "scale_invariant": {
      "type": "boolean",
      "description": "Indicates whether `font_size` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)"
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Color of the text",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "text": {
      "type": "string",
      "description": "Text"
    }
  }
};

export const TriangleListPrimitive = {
  "title": "foxglove.TriangleListPrimitive",
  "description": "A primitive representing a set of triangles or a surface tiled by triangles",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "pose": {
      "title": "foxglove.Pose",
      "description": "Origin of triangles relative to reference frame",
      "type": "object",
      "properties": {
        "position": {
          "title": "foxglove.Vector3",
          "description": "Point denoting position in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x coordinate length"
            },
            "y": {
              "type": "number",
              "description": "y coordinate length"
            },
            "z": {
              "type": "number",
              "description": "z coordinate length"
            }
          }
        },
        "orientation": {
          "title": "foxglove.Quaternion",
          "description": "Quaternion denoting orientation in 3D space",
          "type": "object",
          "properties": {
            "x": {
              "type": "number",
              "description": "x value"
            },
            "y": {
              "type": "number",
              "description": "y value"
            },
            "z": {
              "type": "number",
              "description": "z value"
            },
            "w": {
              "type": "number",
              "description": "w value"
            }
          }
        }
      }
    },
    "points": {
      "type": "array",
      "items": {
        "title": "foxglove.Point3",
        "description": "A point representing a position in 3D space",
        "type": "object",
        "properties": {
          "x": {
            "type": "number",
            "description": "x coordinate position"
          },
          "y": {
            "type": "number",
            "description": "y coordinate position"
          },
          "z": {
            "type": "number",
            "description": "z coordinate position"
          }
        }
      },
      "description": "Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)"
    },
    "color": {
      "title": "foxglove.Color",
      "description": "Solid color to use for the whole shape. One of `color` or `colors` must be provided.",
      "type": "object",
      "properties": {
        "r": {
          "type": "number",
          "description": "Red value between 0 and 1"
        },
        "g": {
          "type": "number",
          "description": "Green value between 0 and 1"
        },
        "b": {
          "type": "number",
          "description": "Blue value between 0 and 1"
        },
        "a": {
          "type": "number",
          "description": "Alpha value between 0 and 1"
        }
      }
    },
    "colors": {
      "type": "array",
      "items": {
        "title": "foxglove.Color",
        "description": "A color in RGBA format",
        "type": "object",
        "properties": {
          "r": {
            "type": "number",
            "description": "Red value between 0 and 1"
          },
          "g": {
            "type": "number",
            "description": "Green value between 0 and 1"
          },
          "b": {
            "type": "number",
            "description": "Blue value between 0 and 1"
          },
          "a": {
            "type": "number",
            "description": "Alpha value between 0 and 1"
          }
        }
      },
      "description": "Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided."
    },
    "indices": {
      "type": "array",
      "items": {
        "type": "integer",
        "minimum": 0
      },
      "description": "Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.\n\nIf omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided)."
    }
  }
};

export const Vector2 = {
  "title": "foxglove.Vector2",
  "description": "A vector in 2D space that represents a direction only",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "x": {
      "type": "number",
      "description": "x coordinate length"
    },
    "y": {
      "type": "number",
      "description": "y coordinate length"
    }
  }
};

export const Vector3 = {
  "title": "foxglove.Vector3",
  "description": "A vector in 3D space that represents a direction only",
  "$comment": "Generated by https://github.com/foxglove/schemas",
  "type": "object",
  "properties": {
    "x": {
      "type": "number",
      "description": "x coordinate length"
    },
    "y": {
      "type": "number",
      "description": "y coordinate length"
    },
    "z": {
      "type": "number",
      "description": "z coordinate length"
    }
  }
};

